
//--- MTF matrix color shades (green/red)

#property indicator_chart_window

extern int    Shift        = -97;
extern int    Spacing      = 20;
extern int    Width        = 19;
extern int    MA_mode      = 0;
input int     Range_in_pips = 40;

#define NSL    8       // number of shade levels
#define NMA    2       // number of MAs, change ma_period[NMA-1] accordingly 
#define NTF    5       // number of time frames/lanes, change tfp[NTF] accordinlgy

double   fml[NTF][NMA];                                    // momentum lines
double   bsl_up[NTF][NSL];                                 // color shades levels to the up side
double   bsl_down[NTF][NSL];                               // color shades levels to the down side
double   bsl[NSL];                                         // color shades distances ('point' adjusted)
double   srl[19];                                          // support/resistance levels (total = 2*NSL + NMA + 1(Bid) )
double   price, point;
double   tmpArray[NMA], lowestMA, highestMA;
int      srgap, tfc, mac;                                  // color shades distances
int      _bsl[NSL]    = {5, 20, 30, 40, 50, 60, 100, 150,}; // time shades distance
int      ma_period[2] = {20,21};         // SMAs, NMA-1
int      tfp[NTF]     = {1,1,1,1,1};                       // Time frames/lanes - in minutes. Like H4=240, D1=1440 etc...
int      counted_bars, startBar, bar, i, y;
int      tf, c, offset, xUnit;
int      shift, spacing, width, supports, resistances;
color    Shade[NSL]    = {C'255,255,255', C'255,200,200', C'245,150,150', C'235,100,100', C'210,20,20', C'180,0,0', C'120,0,0'};
color    Shade1[NSL]    = {C'255,255,255', C'200,255,200', C'150,245,150', C'100,235,100', C'20,210,20', C'0,180,0', C'0,120,0'};
color    col, macolor;
string   objName, dir, MA, name;
string   uniqueid = "PIPdev";
datetime curTime;

double   PipPoints;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   xUnit   = Period() * 60;
   shift   = Shift * xUnit;
   spacing = Spacing * xUnit;
   width   = Width * xUnit;
   point   = 0.01;
   if(Digits > 3) point = 0.0001;
   
   if(Digits==3 || Digits==5)
      {
      PipPoints=Point*10;
      }
   else
      {
      PipPoints=Point;
      }
   
   ChartSetInteger(0,CHART_SCALEFIX,true);
   ChartSetInteger(0,CHART_SHOW_DATE_SCALE,false);
   ChartSetDouble(0,CHART_FIXED_MAX,(Bid+(Range_in_pips*PipPoints)));
   ChartSetDouble(0,CHART_FIXED_MIN,(Bid-(Range_in_pips*PipPoints))); 

//+------------------------------------------------------------------+
//| creating the objects for drawing the grid                        |
//+------------------------------------------------------------------+

   for(i=0; i<NSL; i++)  bsl[i] = _bsl[i]*point;
   for(tf=0; tf<NTF; tf++)
   {
      for(i=0; i<NSL; i++)
      {
         objName = uniqueid + "bsl_up_rect_" + tf + "_" + i;          // upper zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, Shade[i]);
            
         objName = uniqueid + "ext_bsl_up_rect_" + tf;            // external upper zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, C'80,0,0');
            
         objName = uniqueid + "bsl_down_rect_" + tf + "_" + i;        // lower zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, Shade1[i]);
            
         objName = uniqueid + "ext_bsl_down_rect_" + tf;          // external lower zones
         ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, C'0,80,0');
            
         objName = uniqueid+"bsl_up_bar_"+tf+"_"+i;           // upper zone separator lines
         ObjectCreate(objName, OBJ_TREND, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, clrNONE);
            ObjectSet(objName, OBJPROP_RAY, false);
            ObjectSet(objName, OBJPROP_WIDTH, 1);
            ObjectSet(objName, OBJPROP_STYLE, 2);
            
         objName = uniqueid+"bsl_down_bar_"+tf+"_"+i;         // lower zone separator lines
         ObjectCreate(objName, OBJ_TREND, 0, 0, 0, 0, 0);
            ObjectSet(objName, OBJPROP_COLOR, clrNONE);
            ObjectSet(objName, OBJPROP_RAY, false);
            ObjectSet(objName, OBJPROP_WIDTH, 1);
            ObjectSet(objName, OBJPROP_STYLE, 2);

      }

      objName = uniqueid + "MidShade_" + tf;                      // price zone
      ObjectCreate(objName, OBJ_RECTANGLE, 0, 0, 0, 0, 0);
         ObjectSet(objName, OBJPROP_COLOR, C'255,255,255');
         if(tf==0)
         {
         ObjectSet(objName, OBJPROP_COLOR, C'255,255,144');
         }
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+

int deinit()
{
   string lookFor       = uniqueid;
   int    lookForLength = StringLen(lookFor);   
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
   {
      string name = ObjectName(i);
         if(StringSubstr(name, 0, lookForLength) == lookFor) ObjectDelete(name);
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   point   = 0.01;
   if(Digits > 3) point = 0.0001;
   
   ChartSetDouble(0,CHART_FIXED_MAX,(Bid+(Range_in_pips*PipPoints)));
   ChartSetDouble(0,CHART_FIXED_MIN,(Bid-(Range_in_pips*PipPoints)));

//+------------------------------------------------------------------+
//| Time lines                                                       |
//+------------------------------------------------------------------+

   curTime = Time[0];
   price   = Close[0];
    
   for(tf=0; tf<NTF; tf++)
   {
      bar = iBarShift(NULL, tfp[tf], curTime, false);
      for(i=0; i<NMA; i++)  fml[tf][i] = iMA(NULL, tfp[tf], ma_period[i], 0, MA_mode, PRICE_CLOSE, bar/Period());  // populating the fml array...
      for(i=0; i<NMA; i++) tmpArray[i] = fml[tf][i];                                                               // finding the highest and lowest MAs from the (7) fml's/MA's
      ArraySort(tmpArray);                                                                                         // copying the MAs levels of this TF to one dim array to feed ArrayBsearch
         lowestMA  = tmpArray[0];
         highestMA = tmpArray[1];

      for(i=0; i<NSL; i++)                                                                                         // populating the bsl arrays...
      {
         bsl_up[tf][i]   = lowestMA  + bsl[i];
         bsl_down[tf][i] = highestMA - bsl[i];
      }
   }

//+------------------------------------------------------------------+
//| drawing the color shades of the grid from top to bottom          |
//+------------------------------------------------------------------+

   for(tf=0; tf<NTF; tf++) 
   {
      offset = 0;                                                                // find the number of overlapping shades
      for(i=offset; i<NSL-1; i++) if(bsl_up[tf][i] <= bsl_down[tf][i]) offset++;

      if(offset > 0)                                                             // hide first level objects' overlapping, so they wont be left as 'ghosts'
      {
         objName = uniqueid + "bsl_up_rect_" + tf + "_" + (NSL-2-offset+1);
            ObjectSet(objName, OBJPROP_TIME1,  0);
            ObjectSet(objName, OBJPROP_TIME2,  0);
            ObjectSet(objName, OBJPROP_PRICE1, 0);
            ObjectSet(objName, OBJPROP_PRICE2, 0);
            
         objName = uniqueid + "bsl_down_rect_" + tf + "_" + (NSL-2-offset+1);
            ObjectSet(objName, OBJPROP_TIME1,  0);
            ObjectSet(objName, OBJPROP_TIME2,  0);
            ObjectSet(objName, OBJPROP_PRICE1, 0);
            ObjectSet(objName, OBJPROP_PRICE2, 0);

         objName = uniqueid+"bsl_up_bar_"+tf+"_"+(NSL-2-offset+1);
            ObjectSet(objName, OBJPROP_TIME1,  0);
            ObjectSet(objName, OBJPROP_TIME2,  0);
            ObjectSet(objName, OBJPROP_PRICE1, 0);
            ObjectSet(objName, OBJPROP_PRICE2, 0);

         objName = uniqueid+"bsl_down_bar_"+tf+"_"+(NSL-2-offset+1);
            ObjectSet(objName, OBJPROP_TIME1,  0);
            ObjectSet(objName, OBJPROP_TIME2,  0);
            ObjectSet(objName, OBJPROP_PRICE1, 0);
            ObjectSet(objName, OBJPROP_PRICE2, 0);

      }
        
      double highest=0;
      for(i=NSL-2; i>=offset; i--)
      {
         objName = uniqueid + "bsl_up_rect_" + tf + "_" + (i-offset);
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
            ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
            ObjectSet(objName, OBJPROP_PRICE1, bsl_up[tf][i]);
            ObjectSet(objName, OBJPROP_PRICE2, bsl_up[tf][i+1]);
                
         if (bsl_down[tf][i]>highest) highest=bsl_up[tf][i+1];
            
         objName = uniqueid+"bsl_up_bar_"+tf+"_"+(i-offset);
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
            ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
            ObjectSet(objName, OBJPROP_PRICE1, bsl_up[tf][i]);
            ObjectSet(objName, OBJPROP_PRICE2, bsl_up[tf][i]);
            
      }

      double lowest=9999.9999;
      for(i=0; i<NSL-1; i++)
      {
         objName = uniqueid + "bsl_down_rect_" + tf + "_" + (i-offset);
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
            ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
            ObjectSet(objName, OBJPROP_PRICE1, bsl_down[tf][i]);
            ObjectSet(objName, OBJPROP_PRICE2, bsl_down[tf][i+1]);
         
         if(bsl_down[tf][i]<lowest) lowest=bsl_down[tf][i+1];
            
         objName = uniqueid+"bsl_down_bar_"+tf+"_"+(i-offset);
            ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
            ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
            ObjectSet(objName, OBJPROP_PRICE1, bsl_down[tf][i]);
            ObjectSet(objName, OBJPROP_PRICE2, bsl_down[tf][i]);

      }

      objName = uniqueid + "MidShade_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, bsl_up[tf][offset]);
         ObjectSet(objName, OBJPROP_PRICE2, bsl_down[tf][offset]);
      
      objName = uniqueid + "ext_bsl_up_rect_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, highest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMax());
      
      objName = uniqueid + "ext_bsl_down_rect_" + tf;
         ObjectSet(objName, OBJPROP_TIME1,  curTime+shift+tf*spacing);
         ObjectSet(objName, OBJPROP_TIME2,  curTime+shift+tf*spacing+width);
         ObjectSet(objName, OBJPROP_PRICE1, lowest);
         ObjectSet(objName, OBJPROP_PRICE2, WindowPriceMin());

   }
 return(0);
}